#/***************************************************************************
# *   Copyright (C) 2008 by Paul Hardwick                                   *
# *   paul@peck.org.uk                                                      *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU General Public License as published by  *
# *   the Free Software Foundation; either version 2 of the License, or     *
# *   (at your option) any later version.                                   *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU General Public License for more details.                          *
# *                                                                         *
# *   You should have received a copy of the GNU General Public License     *
# *   along with this program; if not, write to the                         *
# *   Free Software Foundation, Inc.,                                       *
# *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
# ***************************************************************************/
"""HSOconsole.py is the main control module.
It provides the program initalisation and control and is the communication link between 
User interface and the serial device."""

__rev__ = "$Revision: 82 $"[10:-1].strip()
__version__ = "1.1."+__rev__
__author__ = "Paul Hardwick <paul@peck.org.uk>"
__date__ =  "$Date: 2008-05-23 18:21:53 +0100 (Fri, 23 May 2008) $"[6:18].strip()


# HSOconsole
# the object that links the GUI and serial stuff
import sys, os, os.path
import userinterface
import ConfigParser
import hsoserial
import time
import signal
import gc

if  os.environ.get("HSOCDEV",False): 
    print "Debug/Test configurations environment"
    configurationdir = "./configuration_src"
else: 
    configurationdir = "/usr/share/HSOconnect"
    
confdir = os.sep.join([configurationdir, "profiles"])
imgdir = os.sep.join([configurationdir, "images"])
langdir = os.sep.join([configurationdir, "languages"])
progconf = os.sep.join([configurationdir, "HSOconnect.cfg"])
logfile =  os.sep.join([configurationdir, "hsoc.log"])
transferlog =  os.sep.join([configurationdir, "transferlog"])
guiconf = "HSOconnect.gui.cfg"
opconf = confdir
toggle = True

def driverReset():
    "A function to reset the HSO driver "
    device = "/sys/bus/usb/drivers/hso/"
    try:
        files = os.listdir(device)
    except:
        return False
    for n in files:
        if ':' in n: 
            print device, n
            os.system("hsolinkcontrol unbind %s %s"%(device, n))
    for n in files:
        if ':' in n: 
            print device, n
            os.system("hsolinkcontrol bind %s %s"%(device, n))
    return True

def driverOK():
    "Perform a simple check and report status of driver"
    device = "/sys/bus/usb/drivers/hso/"
    cnt = 0
    try:
        files = os.listdir(device)
    except:
        #device driver not loaded    
        return None
    for n in files:
        if ":" in n: cnt = cnt + 1
    return cnt

kilo = 1024L
mega = 1048576L
giga = 1073741824L

def stat_format(bytes):
    "Format bytes into bytes, k, M, G."
    if bytes < kilo:
        return "%s bytes"%(bytes, )
    elif bytes < mega:
        return "%.2f Kb"%(float(bytes)/kilo, )
    elif bytes < giga:
        return "%.3f Mb"%(float(bytes)/mega, )
    else:
        return "%.4f Gb"%(float(bytes)/giga, )

def get_connection(pfile):
    "Load up connection details into program configuration disctionary"
    connection = {} #empty dictionary
    default = os.sep.join([confdir, '.default', 'connection'])
    ppp = ConfigParser.ConfigParser()
    try:
        ppp.read(default)
    except ConfigParser.ParsingError , n:
        print n
        return  False  , 'error', 'error_config_connect'      
    try:
        optemp = ['username', 'autodns', 'name', 'ipaddress', 'apn', 'dns1', 'dns2', 'password']
        options = ppp.options("Account")
        for n in optemp:
            if n not in options: 
                print "Connection Configuration file [%s] missing parameter [%s]."%(pfile, n)
                raise IndexError
            connection[n]= ppp.get("Account", n)        
    except:
        return False , 'error', 'error_config_connect'
    ppp = ConfigParser.ConfigParser() #Now load the real connection
    try:
        ppp.read(pfile)
    except ConfigParser.ParsingError , n:
        print n
        return  False  , 'error', 'error_config_connect'      
    try:
        options = ppp.options("Account")
        for n in options:
            connection[n]= ppp.get("Account", n)
        return True, connection
    except:
        return False , 'error', 'error_config_connect'

import re
bytes = re.compile('''^ *(\d+) +\d+ +\d+ +\d+ +\d+ +\d+ +\d+ +\d+ +(\d+)''') 

def get_stats(device = 'hso0'):
    procf = open("/proc/net/dev", 'r')
    stats = procf.readlines()
    procf.close()
    for n in stats:
        if n.find(device) != -1:
            ans =  n[n.find(device)+len(device)+1:]
            if ans:
                ans = bytes.search(ans).groups()
                return int(ans[0]),int(ans[1])
            else:
                return 0,0
                
    else:
        return 0,0
class HSOconnect:
    "The HSOcommect class"
    
    def __init__(self):
        """This module is the hub of the program and is a sort of abstraction layer
        It links the GUI to to the serial isolating the user interface from the main working
        It requires a periodic kick from the GUI to schedule servicing the serial device
        If we ever port to another GUI
        Then we hopefully ;) just need to change the userinterface module
        """
        if not os.path.isdir(configurationdir):
            print "HSOconnect: No configuration data"
            sys.exit(1)
        #Allow us to handle all signals except those that we can't catch
        #Unless specified most cause the app to close cleanly rather than create a 
        #dirty python environment
        for n in range(1, 8)+range(10, 18)+range(20, 30):
            signal.signal(n, self.signal_handler) #install a handler for all unix signals that can be caught

    def run(self):
        "Run the program loop"
        self.prog = {}
        self.restart = False #do we want to restart the app - for loading languages etc.
        self.modemData = ["No Modem Detected"]
        self.str = {} #dictioanry of language strings - passed to UI
        self.ser=None #the serial port object
        self.gui=None #the GUI GTK object
        self.operatorName = ''
        self.techType = ''
        self.registrationState = ''
        #Flags
        self.runfirst = True #setup serial interface
        self.runpin  = self.sendpin = self.initAT = self.prestatus = self.enableconflg = False
        self.running = False #set True when serial fully initialised
        #Check default files exist
        if not os.path.isfile(os.sep.join([langdir, '.default', 'text.cfg'])):
            print "Default text file missing"
            sys.exit(1)
        if not os.path.isfile(os.sep.join([confdir, '.default', 'connection'])):
            print "Default connection file missing"
            sys.exit(1)
        if not os.path.isfile(os.sep.join([confdir, '.default', 'HSOconnect.gui.cfg'])):
            print "Default GUI configuration file missing"
            sys.exit(1)
        
        #Test and load configuration
        if not os.path.isfile(progconf):
            print "Error: Missing Configuration File [%s]"%progconf
            sys.exit(1)
        self.progconfig() #Load program configuration
        self.prog['logfile']=logfile
        #Test and load language
        if self.prog.has_key("language") and len(self.prog['language']) and os.path.isfile(os.sep.join([langdir, self.prog['language'], 'text.cfg'])): 
            self.gettext() #Get language strings
        else:
            print "Error: Missing 'language' key or value. Configuration File [%s]"%progconf
            sys.exit(1)
        #test and load GUI configuration
        if self.prog.has_key("profile") and len(self.prog['profile'])  and os.path.isfile(os.sep.join([confdir, self.prog['profile'], 'HSOconnect.gui.cfg'])): 
            self.gui = self.initGUI(confdir, imgdir, self.str, self.prog, langdir)
        else:
            print "Error: Missing 'profile' key in or value. Configuration File [%s]"%progconf
            sys.exit(1)
        #test and load connection
        if os.path.isfile(os.sep.join([confdir, self.prog['profile'], 'connection'])):
            ret = get_connection(os.sep.join([confdir, self.prog['profile'], 'connection']))
            if ret[0]:
                self.prog['connection'] = ret[1]
            else:
                print "Error in 'connection' file in profile [%s]"%os.sep.join([confdir, self.prog['profile'], 'connection'])
                sys.exit(1)
        else:
            print "Missing 'connection' file in profile [%s]"%os.sep.join([confdir, self.prog['profile'], 'connection'])
            sys.exit(1)
        self.ser = self.initHSOSerial()
        if self.ser and self.gui:
            self.connectElements()
        else:
            print "Failed to create GUI or Serial objects"
            sys.exit(1)
        #get last running total
        self.rxbase,self.txbase = get_stats("hso"+self.prog['procdev'])
        self.transferLimit = int(self.prog.get('transfer_limit','100'))*1024*1024
        self.limitWarningDone = False #only warn once each session
        self.runningTotal = 0
        try:
            tf = open(transferlog,'r')
            self.totalPrevTransfer = int(tf.read().strip())
            tf.close()
        except:
            self.totalPrevTransfer = 0
        userinterface.run()
        del self.gui #force garbage collection
        del self.ser
        gc.collect()
        return self.restart

    def signal_handler(self, sig, frame):
        "A Linux system signal handler - USRSIG1 is used to stop a connection -useful for standby/hibernation"
        if sig == signal.SIGCHLD:
            #we have beed launched as detected on eeePC simpleui launcher
            return
        print "signal break:", sig
        if sig == signal.SIGUSR1:
            #This will be used for suspend/resume signalling
            #Close down connections and reopen application
            #waiting for serial ports to appear
            print "Restarting"
            self.restart = True
            #sys.exit(3)
            self.gui.doTerminate(True)
            return
        else:
            #Other signals received will terminate the program
            self.gui.doTerminate(True)
            self.restart = False
        sys.exit(3)

    def initGUI(self, theConfdir, theImagedir, theStrings, theProgconf, theLangdir):
        "Call the GUI initialisation method"
        gui = userinterface.HSOGUI(theConfdir, theImagedir, theStrings, theProgconf, theLangdir, self.close)
        return gui

    def initHSOSerial(self):
        "Calll the Serial initialisation method"
        return hsoserial.HSOserial(self.prog['port'], 
                                   self.getpin, self.prog, self.cprint)


    def connectElements(self):
        "Link gui, this module and serial interface"
        self.ser.setStatusCallbacks(self.sig, self.operator, 
                                    self.registration, self.tech, 
                                    self.connected, self.disconnected, 
                                    self.received, self.sent, 
                                self.showNetworks)
        self.ser.dialogs(self.WarningDialog, self.ErrorDialog)
        self.gui.schedule(111, self.periodic)
        self.ser.cprint = self.cprint
        self.gui.connect(self.connect) 
        self.gui.currentMode = self.currentMode
        self.gui.setMode = self.setMode
        self.gui.scanNetworks = self.scanNetworks
        self.gui.autoNetwork = self.autoNetwork
        self.gui.setNetwork = self.setNetwork
        self.allowconnect =self.gui.allowconnect
        self.gui.getInfo = self.getInfo
        self.gui.updateConfig = self.updateConfigAndRestart
        self.gui.checkConnect = self.checkConnect
        self.gui.saveconnection = self.saveconnection
        self.gui.copyProfile = self.copyProfile
        self.gui.resetTransfer = self.resetTransfer
        self.gui.doCommand = self.doCommand
        return

    def progconfig(self):
        "Load up the program configuration from external file"
        progcfg = ConfigParser.ConfigParser()
        try:
            progcfg.read(progconf)
        except ConfigParser.ParsingError , n:
            print n
            sys.exit(1)
        for opt in progcfg.options('Config'):
            self.prog[opt] = progcfg.get("Config", opt)

    def gettext(self):
        "Load language strings in to string dictionary"
        textcfg = ConfigParser.ConfigParser()
        try:
            textcfg.read(os.sep.join([langdir, '.default', 'text.cfg']))
        except ConfigParser.ParsingError , n:
            print n
            sys.exit(1)
        for opt in textcfg.options('Strings'):
            self.str[opt] = textcfg.get("Strings", opt) #load up with defaults first
        textcfg = ConfigParser.ConfigParser()
        try:
            textcfg.read(os.sep.join([langdir, self.prog['language'], 'text.cfg']))
        except ConfigParser.ParsingError , n:
            print n
            sys.exit(1)
        for opt in textcfg.options('Strings'):
            self.str[opt] = textcfg.get("Strings", opt)

        self.reg_status = {"0":self.str['reg00'], "1":self.str['reg01'], 
                            "2":self.str['reg02'], "3":self.str['reg03'], 
                            "4":self.str['reg04'], "5":self.str['reg05'], }


    def cprint(self, *args):
        "Console printing to debug window"
        for n in args:
            if str(n).replace("'", "").strip() == '':continue
            if str(n) in "$%#": n= '\r'
            n = str(n).replace('\\n', '').replace('\\r', '\r')
            self.gui.debug(str(n))
            
    def writelog(self):
        rx,tx = get_stats("hso"+self.prog['procdev'])
        rx = rx - self.rxbase
        tx = tx - self.txbase
        log = "%s - %5.1f mins - Total: %s Rx: %sb Tx: %sb [%s %s %s]\n"%(time.asctime(time.localtime(self.connectionStart)),
                                                                                                                    (time.time()-self.connectionStart)/60.0,
                                                                                                                    stat_format(rx+tx),rx,tx, self.operatorName,
                                                                                                                    self.registrationState, self.techType)
        f = open (self.prog['logfile'],'a')
        f.write(log)
        f.close()
        tf = open(transferlog,'w')
        tf.write(str(self.runningTotal))
        tf.close()

        
    def resetTransfer(self):
        #reset running total file and make a note in the log
        log = "Total transfered: %s.\nTotal Reset %s.\n"%(stat_format(self.runningTotal), time.asctime(time.localtime()))
        f = open (self.prog['logfile'],'a')
        f.write(log)
        f.close()
        tf = open(transferlog,'w')
        tf.write('')
        tf.close()
        self.totalPrevTransfer = self.runningTotal = 0
        self.rxbase,self.txbase = get_stats("hso"+self.prog['procdev'])
        
    def if_stat(self,device = 'hso0'):
        """report Tx and Rx data transferred
        uses proc/net/dev which is not reset until reboot
        does not require special permission to read file"""
        rx,tx = get_stats(device)
        rx = rx - self.rxbase
        tx = tx - self.txbase
        self.runningTotal = rx+tx+self.totalPrevTransfer
        #print rx,tx,self.totalPrevTransfer,self.runningTotal
        if self.runningTotal >= self.transferLimit and not self.limitWarningDone: 
            self.WarningDialog('warning','warning_transfer_limit')
            self.limitWarningDone = True
        return  stat_format(tx),  stat_format(rx), stat_format(self.runningTotal)
########################################
#From the gui to main
########################################            
    def updateConfig(self, item, value):
        "Update configuration item in external file"
        f = open(progconf, 'r')
        lines = f.readlines()
        f.close()
        for n in range(len(lines)):
            if lines[n].strip().find(item) == 0:
                lines[n] = "%s : %s\n"%(item, value)
                f = open(progconf, 'w')
                for n in lines:
                    f.write(n)
                f.close()
                break
        else:
            line = "%s : %s\n"%(item, value)
            f = open(progconf, 'a')
            f.write(line)
            f.close()        

    def updateConfigAndRestart(self, conf):
        "Update configuration and restart application"
        for n in conf:
            self.updateConfig(n, conf[n])
        self.restart = True
        self.gui.quit()

    def saveconnection(self, connection):
        "Save edited connection data in connection file"
        for n in connection:
            self.prog['connection'][n] = connection[n]
        #update connection items
        fn = os.sep.join([confdir, self.prog['profile'], 'connection'])
        f = open(fn, 'r')
        lines = f.readlines()
        f.close()
        for item in connection:
            for n in range(len(lines)):
                if (lines[n].strip().lower()).find(item) == 0:
                    lines[n] = "%s : %s\n"%(item, connection[item])
                    break
            else:
                lines.append("%s : %s\n"%(item, connection[item]))
        f = open(fn, 'w')
        for n in lines:
            f.write(n)
        f.close()

    def copyProfile(self, profile):
        "Copy the current profile directory"
        import shutil
        src = os.sep.join([confdir, self.prog['profile']])
        dst = os.sep.join([confdir, profile])
        shutil.copytree(src, dst)
        fn = os.sep.join([confdir, profile, 'connection'])
        f = open(fn, 'r')
        lines = f.readlines()
        f.close()
        for n in range(len(lines)):
            if (lines[n].strip().lower()).find('name') == 0:
                lines[n] = "%s : %s\n"%('name', profile)
                break
        f = open(fn, 'w')
        for n in lines:
            f.write(n)
        f.close()    
########################################
#From the gui to serial
########################################            

    def doCommand(self,cmd):
        "Parse commands for the serial interface"
        if '|' in cmd:
            cmds = cmd.strip().split('|')
            for n in cmds:
                print n
                self.ser.command(n)
        else: 
            print cmd
            self.ser.command(cmd.strip())
        
    def setNetwork(self, net):
        "Network selection GUI sends network code here"
        self.ser.setNetwork(net) #use network code

    def autoNetwork(self):
        "GUI Menu option to trigger automatic network selection"
        self.ser.autoNetwork()

    def connect(self):
        "Start/stop a connection attempt..."
        self.ser.connect()
        
    def setMode(self, mode):
        "Sset the 3g mode"
        if mode == None : return
        self.ser.setMode(mode)

    def close(self):
        "General serial close down"
        if self.ser and self.ser.isOpen():
            if self.ser.dis_connect():
                self.writelog()
            self.ser.closeport()
            self.ser=None

    def scanNetworks(self):
        "request a list of networks/operators"
        self.ser.scanNetworks()

    def getInfo(self):
        "Return some modem device details"
        return ( __version__, __date__, hsoserial.__version__, hsoserial.__date__), self.modemData


########################################
#From the serial interface to the gui
########################################            

    def showNetworks(self, ops):
        """format the list of networks and launch GUI dialog"""
        #['2, "vodafone UK", "voda UK", "23415", 0', '1, "vodafone UK", "voda UK", "23415", 2', '3, "3 UK", "3 UK", "23420", 2', '3, "O2 - UK", "O2 - UK", "23410", 0', '3, "Orange", "Orange", "23433", 0', '3, "T-Mobile UK", "TMO UK", "23430", 0']
        ops.sort() #put the list in order from available to forbidden
        self.ops = {}
        current = -1 #if no network selected
        for n in range(len(ops)):
            parts = ops[n].split(',')
            if parts[0] == '1': 
                perm = self.str['network_status_available']
            elif parts[0] == '2': 
                perm = self.str['network_status_current']
                current = n
            elif parts[0] == '3': 
                perm = self.str['network_status_forbidden']
            name = parts[1].strip('"').upper()
            code = parts[3].strip('"')
            if parts[4] == '0': 
                ctype = self.str['code0']
            elif parts[4] == '2': 
                ctype = self.str['code2']
            else : 
                ctype = self.str['code1']
            self.ops[n] = [name, ctype, perm, code]        
        self.gui.showNetworks(self.ops, current)    #gui returns with index to setNetwork
        

    def WarningDialog(self, text1, text2):
        "translate generic string keys to langauge specific strings and display GUI Warning Dialog"
        self.gui.WarningDialog(self.str[text1], self.str[text2])

    def ErrorDialog(self, text1, text2):
        "translate generic string keys to language specific strings and display GUI Error Dialog        "
        if True:
        #try: 
            self.gui.ErrorDialog(self.str[text1], self.str[text2])
        else:
        #except:
            print "Unable to show GUI Erorrs: ", text1, text2
       
    
    def sig(self, value):
        "Update GUI with new signal strength, GUI assumes an integer valsue 0-32"
        self.gui.doSignal(int(value))

    
    def received(self, value):
        "Update GUI with received amount"
        self.gui.showRxData(value)        
        
    def sent(self, value):
        "Update GUI with transmitted amount"
        self.gui.showTxData(value)
        
    def operator(self, value):
        "Update GUI with operator name"
        self.operatorName = value
        self.gui.operator(self.operatorName)


    def registration(self, value):
        "Update registration state and disable GUI connect if not registered"
        if int(value) == 1 or int(value) == 5 : self.allowconnect(True)
        else: self.allowconnect(False)
        if (value == 5) and  (self.prog.get('warn_roaming','0') == '1'):
            self.WarningDialog(self.str['warning'], self.str['warning_roam'])
        self.registrationState = self.reg_status[str(value)]
        self.gui.registration( self.registrationState)
    
    def tech(self, wcdma, value):
        "Update GUI with current technology status"
        if wcdma:
            techtype = chr(ord('g')+int(value))
        else:
            techtype = chr(ord('a')+int(value))
        #translate the 2g/3g status
        self.techType = self.str["code-"+techtype]
        self.gui.tech(self.techType)
   
    def connected(self):
        "Update GUI to connected state"
        self.connectionStart = time.time()
        self.rxbase,self.txbase = get_stats("hso"+self.prog['procdev'])
        try:
            tf = open(transferlog,'r')
            self.totalPrevTransfer = int(tf.read().strip())
            tf.close()
        except:
            self.totalPrevTransfer = 0        
        self.runningTotal = self.totalPrevTransfer
        self.cprint("\n======connected======\n")
        self.gui.showconnected()

        
    def disconnected(self):
        "Update GUI discinnected state"
        self.writelog()
        self.cprint("\n======disconnected======\n")
        self.gui.showdisconnected()

    def getpin(self):
        "Serial routines require pin number so setup up pin dialog request."
        return self.gui.getpin(self.str['pin_title'], self.str['pin_prompt'])
    

        
    def currentMode(self):
        "Return the 2g/3g mode"
        return self.ser.mode
    
    def runSerial(self):
        "Called by the periodic routine during device init"
        if not self.ser.isOpen():
            ret = self.ser.initserial()
            if ret: print ret
            if ret == -1:
                print "port open but no dialog"
                driverReset()
                time.sleep(0.1)
                ret = self.ser.initserial()
            if ret == None:
                self.runfirst = True #keep polling for a port - self.periodic will call again
                return False  # return if no port detected
        ret = self.ser.checkpin() #Do we need a pin?
        if ret == "SIM PIN": #pin required
            self.runpin = True # Telll periodic to run the pin dialog section
            return True
        elif ret == None:
            #SIM problem
            self.gui.ErrorDialog(self.str['error'], self.str['error_checksim'])
            sys.exit(1)            
        elif ret in ['SIM PUK', 'SIM PUK2']:
            print "SIM Needs PUK or PUK2"
            #extra PINS needed :(
            #We dont handle PUK or PUK2 we just tell the user the problem and exit
            if ret == 'SIM PUK':
                self.gui.ErrorDialog(self.str['error'], self.str['error_pin_locked_puk'])
            else:
                self.gui.ErrorDialog(self.str['error'], self.str['error_pin_locked_puk2'])
            sys.exit(1)
        elif ret in ["ERROR", "NoSIM"]:
            print "Missing SIM or Error"
            #SIM problem - SIM missing or disabled/faulty
            self.gui.ErrorDialog(self.str['error'], self.str['error_checksim'])
            sys.exit(1)
        elif ret == 'READY':
            return True
        else: 
            print "SIM response:", ret
            self.gui.ErrorDialog(self.str['error'], self.str['error_checksim'])
            sys.exit(1)


    def dopin(self):
        "Called if pin dialog needed - broken up to allow GUI to refresh"
        pin = self.getpin()
        if pin == "":
            self.gui.ErrorDialog(self.str['error'], self.str['error_nopin'])
            sys.exit(1)
        return pin
         
    def checkConnect(self):
        "Report the connection statistics"
        if True:
            return self.if_stat("hso"+self.prog['procdev'])
        else:
            if self.ser.isOpen() : print "Serial port open but failed to read /proc"
        return -1, -1,stat_format(self.totalPrevTransfer)
 
    def periodic(self):
        """This routine is called by the GUI scheduler.
        to keep the GUI refreshed this routine must return regularly
        We break up the device initilisation to do this."""
        if self.running:
           #The normal periodic service ...    
            if self.prog['debug']=='1': self.gui.status1.set_text("Tick: %s"%time.time())
            try:
                if not self.ser.status(): raise IndexError #need better handling here!!!
            except:
                #self.ErrorDialog('error', 'error_comms_restart')
                self.restart = True
                self.gui.quit()    
            return True
            
        #use the peridoc to init the serial device    
        elif self.runfirst: # Try to open port - this section is done until a port is detected
            if self.runSerial():
                self.runfirst = False
                self.sendpin = False
                self.initAT = True
            else:
                ok = driverOK()
                if ok and ok == 2:
                    #driverReset()
                    time.sleep(0.1)
                elif ok and ok >0:
                    driverReset()
                    print "driver not finished loading yet?"
            return True
        elif self.runpin: #
            #make sure pin dialog is cleared away
            self.pin = self.dopin()
            self.runpin = False
            self.sendpin = True
            return True
        elif self.sendpin:
            #process the pin on the next periodic
            ret = self.ser.sendpin(self.pin)
            if ret == False:
                self.gui.WarningDialog(self.str['error'], self.str['error_badpin'])           
            self.runfirst = True
            self.sendpin = False
            self.initAT = True
            return True
        elif self.initAT:
            self.modemData = self.ser.initAT()
            self.initAT=False
            self.prestatus = True
            return True
        elif self.prestatus:
            self.ser.pre_status()
            self.gui.showmenu()
            self.prestatus = False
            self.enableconflg = True
            self.ser.requestStatus()
            return True
        elif self.enableconflg:
            self.gui.enableconnect()
            self.enableconflg = False
            self.running = True
            return True 
        
def run():
    "Called by HSOconnect to start the main program loop"
    while True:
        hsc = HSOconnect()
        ans = hsc.run()
        del hsc
        if ans == False:  break
        
        

    
if __name__ == "__main__":
    run()

    
